from click.exceptions import ClickException


class NoInternetException(ClickException):
    def __init__(self):
        super(NoInternetException, self).__init__(
            'Could not connect to server')


class CLIRequiredException(ClickException):
    def __init__(self, field_name):
        super(CLIRequiredException, self).__init__(
            'Option %s is required' % field_name)


class NotStolosDirectoryException(ClickException):
    def __init__(self):
        super(NotStolosDirectoryException, self).__init__(
            'Current directory is not a Stolos-enabled directory.')


class UnknownError(ClickException):
    def __init__(self, status_code, text):
        super(UnknownError, self).__init__(
            'Unknown error.\nStatus code: {status_code}\n{text}'.format(
                status_code=status_code, text=text))


class ServerError(ClickException):
    def __init__(self, text):
        super(ServerError, self).__init__('Server error:\n{}'.format(text))


class Unauthorized(ClickException):
    def __init__(self, result):
        super(Unauthorized, self).__init__(
            'Unauthorized: {}'.format(result['detail']))


class BadRequest(ClickException):
    def __init__(self, result):
        errors = {key: ','.join(result[key]) for key in result}
        message = errors.pop('non_field_errors', '')
        errors_disp = ['{key}: {error}'.format(key=key, error=errors[key])
                       for key in errors]
        super(BadRequest, self).__init__(
            'Bad request: {message}\n{errors}'.format(
                message=message, errors='\n'.join(errors_disp)))

class Timeout(ClickException):
    def __init__(self):
        super(Timeout, 'Request timed out')


class ResourceDoesNotExist(ClickException):
    def __init__(self, result):
        super(ResourceDoesNotExist, self).__init__(
            'Resource does not exist: {}'.format(result))


class NotLoggedInException(ClickException):
    def __init__(self):
        super(NotLoggedInException, self).__init__(
            'You are not logged in, please run stolos login first.')


class ResourceAlreadyExists(ClickException):
    def __init__(self):
        super(ResourceAlreadyExists, self).__init__('Resource already exists.')



"""
Helper functions for detecting the user's current shell.
"""
import os
import os.path

import click


ENV_TEMPLATE = """
{env_vars}\
{usage_hint}
"""


def detect():
    """
    Detects the shell the user is currently using. The logic is picked from
    Docker Machine
    https://github.com/docker/machine/blob/master/libmachine/shell/shell.go#L13
    """
    shell = os.getenv('SHELL')
    if not shell:
        return None
    if os.getenv('__fish_bin_dir'):
        return 'fish'
    return os.path.basename(shell)


def shell_config(shell):
    """
    Returns a dict in the following form, depending on the given shell:
    return {
        'prefix': '<preffix-to-use>',
        'suffix': 'suffix-to-use',
        'delimiter': '<delimiter-to-use>',
    }
    Logic from Docker Machine:
    https://github.com/docker/machine/blob/master/commands/env.go#L125
    """
    if shell == 'fish':
        return {
            'prefix': 'set -gx ',
            'suffix': '";\n',
            'delimiter': ' "',
        }

    elif shell == 'powershell':
        return {
            'prefix': '$Env:',
            'suffix': '"\n',
            'delimiter': ' = "',
        }

    elif shell == 'cmd':
        return {
            'prefix': 'SET ',
            'suffix': '\n',
            'delimiter': '=',
        }

    elif shell == 'tcsh':
        return {
            'prefix': 'setenv ',
            'suffix': '";\n',
            'delimiter': ' "',
        }

    elif shell == 'emacs':
        return {
            'prefix': '(setenv "',
            'suffix': '")\n',
            'delimiter': '" "',
        }

    else:
        return {
            'prefix': 'export ',
            'suffix': '"\n',
            'delimiter': '="',
        }


def usage_hint(command, shell):
    """
    Returns the usage hint and comment, for the given shell.
    """
    if shell == 'fish':
        return ('eval ({command})'.format(command=command), '#')
    elif shell == 'powershell':
        return ('{command} | ForEach-Object {{If (-Not[string]::IsNullOrEmpty($_)) {{ $_ | Invoke-Expression }}}}'.format(command=command), '#')
    elif shell == 'cmd':
        return ('\t@FOR /f "tokens=*" %i IN ({command}) DO @%i'.format(
            command=command), 'REM')
    elif shell == 'emacs':
        return ('(with-temp-buffer (shell-command {command} (current-buffer))'
                '(eval-buffer))'.format(command=command), ';;')
    elif shell == 'tcsh':
        return ('eval `{command}`'.format(command=command), ':')
    else:
        return ('eval $({command})'.format(command=command), '#')


def print_env_eval(command, shell, env_dict):
    """
    Prints the environment evaluation snippet, given a shell and an environment
    dict.
    """
    if not shell:
        shell = detect()
    if not shell:
        click.echo(
            'Shell could not be detected, falling back to bash', err=True)
    config = shell_config(shell)
    hint, comment = usage_hint(command, shell)
    uhint = '{} Run this command to configure your shell: \n{} {}\n'.format(
        comment, comment, hint)
    config['usage_hint'] = uhint
    env_vars = ''
    for key in sorted(env_dict.keys()):
        config['key'] = key
        config['val'] = env_dict[key]
        env_vars = env_vars + '{prefix}{key}{delimiter}{val}{suffix}'.format(**config)
    config['env_vars'] = env_vars
    msg = ENV_TEMPLATE.format(**config)
    click.echo(msg)



from setuptools import setup, find_packages
from stolos import VERSION


setup(
    name='stolosctl',
    version=VERSION,
    author='SourceLair PC',
    url='https://www.sourcelair.com/',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click >= 6.0',
        'PyYAML >= 3.11',
        'tabulate >= 0.7.5',
        'requests >= 2.6.1, < 2.12',
    ],
    extras_require={
        'security': ['requests[security] >= 2.6.1, < 2.12'],
        'compose': ['docker-compose >= 1.7.0'],
    },
    entry_points='''
    [console_scripts]
    stolos=stolos.cli:cli
    stolosctl=stolos.cli:cli
    ''',
)
